var chai = require('chai');
var expect = chai.expect;

import { default as reduxAutomapES6, automap, merge } from './redux-automap';

const reduxAutomapCJS = require('./redux-automap');
const automapCJS = reduxAutomapCJS.automap;
const mergeCJS = reduxAutomapCJS.merge;

// SAMPLE CONFIG
let todosState = [];
let actionReducers = [
  {
    addTodo: (text) => ({ type: 'ADD_TODO', text }),
    reducer: (state, action) => [ ...state, action.text]
  },
  {
    clearTodos: () => ({ type: 'CLEAR_TODOS' }),
    reducer: (state, action) => []
  }
];
let sampleConfig = { namespace: 'foo', initialState: todosState, actionReducers };
// END: SAMPLE CONFIG

describe('"require" vs "import"', function() {
  it('both return same default', function() {
    expect(typeof reduxAutomapES6.automap).to.equal('function');
    expect(typeof reduxAutomapCJS.automap).to.equal('function');
  });

  it('both return named exports', function() {
    expect(typeof automap).to.equal('function');
    expect(typeof automapCJS).to.equal('function');
  });

  it('both automap functions function minimally', function() {
    let config = { namespace: 'foo' };
    let mappedES6 = automap(config);
    let mappedCJS = automapCJS(config);

    expect(mappedCJS.namespace).to.equal(mappedES6.namespace);
  });
});

describe('automap({ namespace, actionReducers, initialState })', function() {
  let todos = automap(sampleConfig);

  it('reducer throws when no action passed', function() {
    expect(todos.reducer).to.throw();
  });

  it('reducer delivers initialState if no matching action found', function() {
    expect(todos.reducer(undefined, {})).to.equal(todosState);
  });

  it('exports actions approporiately', function() {
    expect(typeof todos.actions.addTodo).to.equal('function');
  });

  it('actions creators create action objects', function() {
    expect(todos.actions.addTodo('miffles')).to.eql({ type: 'ADD_TODO', text: 'miffles' });
  });

  it('reducer works as intended', function() {
    let { addTodo, clearTodos } = todos.actions;
    let miffles = addTodo('miffles');
    let vlad = addTodo('vlad');
    let clearAction = clearTodos();
    let state = todosState;

    state = todos.reducer(state, miffles);
    expect(state).to.eql(['miffles']);

    state = todos.reducer(state, vlad);
    expect(state).to.eql(['miffles', 'vlad']);

    state = todos.reducer(state, clearAction);
    expect(state).to.eql([]);
  });
});

describe('merge([ maps ])', function() {
  let todos = automap(sampleConfig);
  let merged = merge([ todos ]);
  console.log('merged', merged);

  it('merges maps into { namespace: reducer(state, action) } format for use in rootReducer/combineReducers', function() {
    expect(typeof merged.foo).to.equal('function');
  });
});
